@@ -413,36 +413,6 @@ eel_gtk_window_set_initial_geometry_from_string (GtkWindow *window,
 	eel_gtk_window_set_initial_geometry (window, geometry_flags, left, top, width, height);
 }
 
-/**
- * eel_gtk_selection_data_copy_deep:
- * 
- * Copies a GtkSelectionData, and copies the data too.
- * @data: The GtkSelectionData to be copied.
- **/
-GtkSelectionData *
-eel_gtk_selection_data_copy_deep (const GtkSelectionData *data)
-{
-	GtkSelectionData *copy;
-
-	copy = g_new0 (GtkSelectionData, 1);
-	gtk_selection_data_set (copy, data->type, data->format, data->data, data->length);
-
-	return copy;
-}
-
-/**
- * eel_gtk_selection_data_free_deep:
- * 
- * Frees a GtkSelectionData, and frees the data too.
- * @data: The GtkSelectionData to be freed.
- **/
-void
-eel_gtk_selection_data_free_deep (GtkSelectionData *data)
-{
-	g_free (data->data);
-	gtk_selection_data_free (data);
-}
-
 /**
  * eel_gtk_signal_connect_free_data:
  * 
@@ -609,56 +579,6 @@ eel_point_in_widget (GtkWidget *widget,
 	return eel_point_in_allocation (&widget->allocation, x, y);
 }
 
-/**
- * eel_gtk_object_list_ref
- *
- * Ref all the objects in a list.
- * @list: GList of objects.
- **/
-GList *
-eel_gtk_object_list_ref (GList *list)
-{
-	g_list_foreach (list, (GFunc) g_object_ref, NULL);
-	return list;
-}
-
-/**
- * eel_gtk_object_list_unref
- *
- * Unref all the objects in a list.
- * @list: GList of objects.
- **/
-void
-eel_gtk_object_list_unref (GList *list)
-{
-	g_list_foreach (list, (GFunc) g_object_unref, NULL);
-}
-
-/**
- * eel_gtk_object_list_free
- *
- * Free a list of objects after unrefing them.
- * @list: GList of objects.
- **/
-void
-eel_gtk_object_list_free (GList *list)
-{
-	eel_gtk_object_list_unref (list);
-	g_list_free (list);
-}
-
-/**
- * eel_gtk_object_list_copy
- *
- * Copy the list of objects, ref'ing each one.
- * @list: GList of objects.
- **/
-GList *
-eel_gtk_object_list_copy (GList *list)
-{
-	return g_list_copy (eel_gtk_object_list_ref (list));
-}
-
 /**
  * eel_gtk_widget_set_shown
  *
@@ -867,61 +787,6 @@ eel_gtk_signal_connect_while_realized (GtkObject *object,
 						 name, callback, callback_data);
 }
 
-/**
- * eel_nullify_when_destroyed.
- *
- * Nulls out a saved reference to an object when the object gets destroyed.
- * @data: Address of the saved reference.
- **/
-
-void 
-eel_nullify_when_destroyed (gpointer data)
-{
-	GObject **object_reference;
-
-	object_reference = (GObject **)data;
-	if (*object_reference == NULL) {
-		/* the reference is  NULL, nothing to do. */
-		return;
-	}
-
-	g_assert (G_IS_OBJECT (*object_reference));
-
-	g_object_add_weak_pointer (*object_reference,
-				   (void*) object_reference);
-}
-
-/**
- * eel_nullify_cancel.
- *
- * Disconnects the signal used to make eel_nullify_when_destroyed.
- * Used when the saved reference is no longer needed, the structure it is in is
- * being destroyed, etc. Nulls out the refernce when done.
- * @data: Address of the saved reference.
- **/
-
-void 
-eel_nullify_cancel (gpointer data)
-{
-	GObject **object_reference;
-
-	object_reference = (GObject **)data;	
-	if (*object_reference == NULL) {
-		/* the object was already destroyed and the reference nulled out,
-		 * nothing to do.
-		 */
-		return;
-	}
-
-	g_assert (G_IS_OBJECT (*object_reference));
-
-	g_object_remove_weak_pointer (*object_reference,
-				      (void*) object_reference);
-	
-	*object_reference = NULL;
-}
-
-
 /**
  * eel_gtk_container_get_first_child.
  *
@@ -1113,323 +978,6 @@ eel_gtk_widget_find_windowed_ancestor (GtkWidget *widget)
 	return widget;
 }
 
-
-
-/*following code shamelessly stolen from gtk*/
-static void
-rgb_to_hls (gdouble *r,
-            gdouble *g,
-            gdouble *b)
-{
-	gdouble min;
-	gdouble max;
-	gdouble red;
-	gdouble green;
-	gdouble blue;
-	gdouble h, l, s;
-	gdouble delta;
-
-	red = *r;
-	green = *g;
-	blue = *b;
-
-	if (red > green) {
-		if (red > blue)
-			max = red;
-		else
-			max = blue;
-		
-		if (green < blue)
-			min = green;
-		else
-			min = blue;
-	} else {
-		if (green > blue)
-			max = green;
-		else
-			max = blue;
-		
-		if (red < blue)
-			min = red;
-		else
-			min = blue;
-	}
-
-	l = (max + min) / 2;
-	s = 0;
-	h = 0;
-
-	if (max != min) {
-		if (l <= 0.5)
-			s = (max - min) / (max + min);
-		else
-			s = (max - min) / (2 - max - min);
-		
-		delta = max -min;
-		if (red == max)
-			h = (green - blue) / delta;
-		else if (green == max)
-			h = 2 + (blue - red) / delta;
-		else if (blue == max)
-			h = 4 + (red - green) / delta;
-		
-		h *= 60;
-		if (h < 0.0)
-			h += 360;
-	}
-	
-	*r = h;
-	*g = l;
-	*b = s;
-}
-
-static void
-hls_to_rgb (gdouble *h,
-            gdouble *l,
-            gdouble *s)
-{
-	gdouble hue;
-	gdouble lightness;
-	gdouble saturation;
-	gdouble m1, m2;
-	gdouble r, g, b;
-	
-	lightness = *l;
-	saturation = *s;
-
-	if (lightness <= 0.5)
-		m2 = lightness * (1 + saturation);
-	else
-		m2 = lightness + saturation - lightness * saturation;
-	m1 = 2 * lightness - m2;
-
-	if (saturation == 0) {
-		*h = lightness;
-		*l = lightness;
-		*s = lightness;
-	} else {
-		hue = *h + 120;
-		while (hue > 360)
-			hue -= 360;
-		while (hue < 0)
-			hue += 360;
-		
-		if (hue < 60)
-			r = m1 + (m2 - m1) * hue / 60;
-		else if (hue < 180)
-			r = m2;
-		else if (hue < 240)
-			r = m1 + (m2 - m1) * (240 - hue) / 60;
-		else
-			r = m1;
-
-		hue = *h;
-		while (hue > 360)
-			hue -= 360;
-		while (hue < 0)
-			hue += 360;
-		
-		if (hue < 60)
-			g = m1 + (m2 - m1) * hue / 60;
-		else if (hue < 180)
-			g = m2;
-		else if (hue < 240)
-			g = m1 + (m2 - m1) * (240 - hue) / 60;
-		else
-			g = m1;
-		
-		hue = *h - 120;
-		while (hue > 360)
-			hue -= 360;
-		while (hue < 0)
-			hue += 360;
-		
-		if (hue < 60)
-			b = m1 + (m2 - m1) * hue / 60;
-		else if (hue < 180)
-			b = m2;
-		else if (hue < 240)
-			b = m1 + (m2 - m1) * (240 - hue) / 60;
-		else
-			b = m1;
-		
-		*h = r;
-		*l = g;
-		*s = b;
-	}
-}
-
-void
-eel_gtk_style_shade (GdkColor *a,
-			  GdkColor *b,
-			  gdouble   k)
-{
-
-	gdouble red;
-	gdouble green;
-	gdouble blue;
-	
-	red = (gdouble) a->red / 65535.0;
-	green = (gdouble) a->green / 65535.0;
-	blue = (gdouble) a->blue / 65535.0;
-	
-	rgb_to_hls (&red, &green, &blue);
-
-	green *= k;
-	if (green > 1.0)
-		green = 1.0;
-	else if (green < 0.0)
-		green = 0.0;
-
-	blue *= k;
-	if (blue > 1.0)
-		blue = 1.0;
-	else if (blue < 0.0)
-		blue = 0.0;
-
-	hls_to_rgb (&red, &green, &blue);
-
-	b->red = red * 65535.0;
-	b->green = green * 65535.0;
-	b->blue = blue * 65535.0;
-}
-
-/* helper function for eel_get_window_list_ordered_front_to_back () */
-static GtkWidget *
-window_at_or_below (int depth, Window xid, gboolean *keep_going)
-{
-	static Atom wm_state = 0;
-
-	GtkWidget *widget;
-
-	Atom actual_type;
-	int actual_format;
-	gulong nitems, bytes_after;
-	gulong *prop;
-
-	GdkWindow *window;
-	gpointer data;
-
-	Window root, parent, *children;
-	int nchildren, i;
-
-	if (wm_state == 0) {
-		wm_state = XInternAtom (GDK_DISPLAY (), "WM_STATE", False);
-	}
-
-	/* Check if the window is a top-level client window.
-	 * Windows will have a WM_STATE property iff they're top-level.
-	 */
-	if (XGetWindowProperty (GDK_DISPLAY (), xid, wm_state, 0, 1,
-				False, AnyPropertyType, &actual_type,
-				&actual_format, &nitems, &bytes_after,
-				(guchar **) &prop) == Success
-	    && prop != NULL && actual_format == 32 && prop[0] == NormalState)
-	{
-		/* Found a top-level window */
-
-		if (prop != NULL) {
-			XFree (prop);
-		}
-
-		/* Does GDK know anything about this window? */
-		window = gdk_window_lookup (xid);
-		if (window != NULL) {
-			gdk_window_get_user_data (window, &data);
-			if (data != NULL)
-			{
-				/* Found one of the widgets we're after */
-				*keep_going = FALSE;
-				return GTK_WIDGET (data);
-			}
-		}
-
-		/* No point in searching past here. It's a top-level
-		 * window, but not from this application.
-		 */
-		*keep_going = FALSE;
-		return NULL;
-	}
-
-	/* Not found a top-level window yet, so keep recursing. */
-	if (depth < MAXIMUM_WM_REPARENTING_DEPTH) {
-		if (XQueryTree (GDK_DISPLAY (), xid, &root,
-				&parent, &children, &nchildren) != 0)
-		{
-			widget = NULL;
-
-			for (i = 0; *keep_going && i < nchildren; i++) {
-				widget = window_at_or_below (depth + 1,
-							     children[i],
-							     keep_going);
-			}
-
-			if (children != NULL) {
-				XFree (children);
-			}
-
-			if (! *keep_going) {
-				return widget;
-			}
-		}
-	}
-
-	return NULL;
-}
-
-/* eel_get_window_list_ordered_front_to_back:
- *
- * Return a list of GtkWindows's, representing the stacking order (top to
- * bottom) of all windows (known to the local GDK).
- *
- * (Involves a large number of X server round trips, so call sparingly)
- */
-GList *
-eel_get_window_list_ordered_front_to_back (void)
-{
-	Window root, parent, *children;
-	int nchildren, i;
-	GList *windows;
-	GtkWidget *widget;
-	gboolean keep_going;
-
-	/* There's a possibility that a window will be closed in
-	 * the period between us querying the child-of-root windows
-	 * and getting round to search _their_ children. So arrange
-	 * for errors to be caught and ignored.
-	 */
-
-	gdk_error_trap_push ();
-
-	windows = NULL;
-
-	if (XQueryTree (GDK_DISPLAY (), GDK_ROOT_WINDOW (),
-			&root, &parent, &children, &nchildren) != 0)
-	{
-		for (i = 0; i < nchildren; i++) {
-			keep_going = TRUE;
-			widget = window_at_or_below (0, children[i],
-						     &keep_going);
-			if (widget != NULL) {
-				/* XQueryTree returns window in bottom ->
-				 * top order, so consing up the list in
-				 * the normal manner will reverse this
-				 * giving the desired top -> bottom order
-				 */
-				windows = g_list_prepend (windows, widget);
-			}
-		}
-		if (children != NULL) {
-			XFree (children);
-		}
-	}
-
-	gdk_flush ();
-	gdk_error_trap_pop ();
-
-	return windows;
-}
-
 /* eel_gtk_get_system_font:
  *
  * Return the system font as selected in the control center. Need to 
@@ -1455,136 +1003,6 @@ eel_gtk_get_system_font (void)
 	return font;
 }
 
-/**
- * eel_gtk_widget_standard_realize:
- *
- * @widget: A GtkWidget (must be unrealized).
- *
- * A standard implementation of GtkWidget::realize for widgets
- * that need a GdkWindow (GTK_WIDGET_NO_WINDOW (widget) == FALSE)
- * 
- * You should call gtk_widget_set_events() in your "GtkWidget::initialize"
- * method to set the event mask for your widget.
- */
-void
-eel_gtk_widget_standard_realize (GtkWidget *widget)
-{
-	GdkWindowAttr attributes;
-	gint attributes_mask;
-	gint border_width;
-	
-	g_return_if_fail (GTK_IS_WIDGET (widget));
-	g_return_if_fail (!GTK_WIDGET_REALIZED (widget));
-
-	GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
-
-	/* Its a little weird to check for the specific container type
-	 * here, but it makes this function convenient to use for
-	 * for containers as well.
-	 */
-	border_width = 
-		GTK_IS_CONTAINER (widget) ?
-		GTK_CONTAINER (widget)->border_width :
-		0;
-	
-	attributes.x = widget->allocation.x + border_width;
-	attributes.y = widget->allocation.y + border_width;
-	attributes.width = widget->allocation.width - 2 * border_width;
-	attributes.height = widget->allocation.height - 2 * border_width;
-	attributes.window_type = GDK_WINDOW_CHILD;
-	attributes.wclass = GDK_INPUT_OUTPUT;
-	attributes.visual = gtk_widget_get_visual (widget);
-	attributes.colormap = gtk_widget_get_colormap (widget);
-	attributes.event_mask = gtk_widget_get_events (widget);
-	
-	attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
-	
-	widget->window = gdk_window_new (gtk_widget_get_parent_window (widget), &attributes, attributes_mask);
-	gdk_window_set_user_data (widget->window, widget);
-	
-	widget->style = gtk_style_attach (widget->style, widget->window);
-	gtk_style_set_background (widget->style, widget->window, GTK_STATE_NORMAL);
-}
-
-/**
- * eel_gtk_bin_standard_size_allocate:
- *
- * @widget: A GtkBin widget.
- * @allocation: GtkAllocation pointer from size_allocate method.
- *
- * An implementation of GtkWidget::size_allocate for GtkBin widgets.
- */
-void
-eel_gtk_bin_standard_size_allocate (GtkWidget *widget,
-				    GtkAllocation *allocation)
-{
-	GtkWidget *child;
-	GtkAllocation child_allocation;
-	
-	g_return_if_fail (GTK_IS_BIN (widget));
-	g_return_if_fail (allocation != NULL);
-
-	child = GTK_BIN (widget)->child;
-	
-	if (child != NULL) {
-		g_return_if_fail (GTK_IS_WIDGET (child));
-	}
-	
-	widget->allocation = *allocation;
-	
-	child_allocation.x = 0;
-	child_allocation.y = 0;
-	child_allocation.width = MAX (allocation->width - GTK_CONTAINER (widget)->border_width * 2, 0);
-	child_allocation.height = MAX (allocation->height - GTK_CONTAINER (widget)->border_width * 2, 0);
-	
-	if (GTK_WIDGET_REALIZED (widget)) {
-		gdk_window_move_resize (widget->window,
-					allocation->x + GTK_CONTAINER (widget)->border_width,
-					allocation->y + GTK_CONTAINER (widget)->border_width,
-					child_allocation.width,
-					child_allocation.height);
-	}
-	
-	if (child != NULL) {
-		gtk_widget_size_allocate (child, &child_allocation);
-	}
-}
-
-/**
- * eel_gtk_bin_standard_size_request:
- *
- * @container: A GtkContainer widget.
- * @requisition: GtkRequisition pointer from size_request method.
- *
- * An implementation of GtkWidget::size_request for GtkBin widgets.
- */
-void
-eel_gtk_bin_standard_size_request (GtkWidget *widget,
-				   GtkRequisition *requisition)
-{
-	GtkRequisition child_requisition;
-	GtkWidget *child;
-	
-	g_return_if_fail (GTK_IS_BIN (widget));
-	g_return_if_fail (requisition != NULL);
-
-	child = GTK_BIN (widget)->child;
-
-	if (child != NULL) {
-		g_return_if_fail (GTK_IS_WIDGET (child));
-	}
-	
-	requisition->width = GTK_CONTAINER (widget)->border_width * 2;
-	requisition->height = GTK_CONTAINER (widget)->border_width * 2;
-	
-	if (child != NULL && GTK_WIDGET_VISIBLE (child)) {
-		gtk_widget_size_request (child, &child_requisition);
-		
-		requisition->width += child_requisition.width;
-		requisition->height += child_requisition.height;
-	}
-}
-
 static PangoContext *
 create_pango_ft2_context (PangoContext *base_context)
 {
@@ -1618,3 +1036,46 @@ eel_gtk_widget_get_pango_ft2_context (GtkWidget *widget)
 	
 	return context;
 }
+
+void
+eel_gtk_widget_get_button_event_location (GtkWidget *widget,
+					  const GdkEventButton *event,
+					  int *x,
+					  int *y)
+{
+	int window_x, window_y;
+
+	g_return_if_fail (GTK_IS_WIDGET (widget));
+	g_return_if_fail (event != NULL);
+
+	gdk_window_get_position (event->window, &window_x, &window_y);
+	if (x != NULL) {
+		*x = event->x + window_x - widget->allocation.x;
+	}
+	if (y != NULL) {
+		*y = event->y + window_y - widget->allocation.y;
+	}
+}
+
+void
+eel_gtk_widget_get_motion_event_location (GtkWidget *widget,
+					  const GdkEventMotion *event,
+					  int *x,
+					  int *y)
+{
+	eel_gtk_widget_get_button_event_location (widget, (const GdkEventButton *) event, x, y);
+}
+
+gboolean
+eel_gtk_tree_view_cell_is_completely_visible (GtkTreeView          *tree_view,
+					      GtkTreePath          *path,
+					      GtkTreeViewColumn    *column)
+{
+	GdkRectangle cell_rect, visible_rect;
+
+	gtk_tree_view_get_background_area (tree_view, path, column, &cell_rect);
+	gtk_tree_view_widget_to_tree_coords (tree_view, cell_rect.x, cell_rect.y,
+					     &cell_rect.x, &cell_rect.y);
+	gtk_tree_view_get_visible_rect (tree_view, &visible_rect);
+	return eel_gdk_rectangle_contains_rectangle (visible_rect, cell_rect);
+}
