@@ -35,7 +35,6 @@
 #include <gconf/gconf-client.h>
 #include <gconf/gconf.h>
 #include <gtk/gtksignal.h>
-#include <libgnome/gnome-i18n.h>
 
 /* An enumeration used for updating auto-storage variables in a type-specific way. 
  * FIXME: there is another enumeration like this in eel-global-preferences.c,
@@ -339,6 +338,83 @@ eel_preferences_get_integer (const char *name)
 	return result;
 }
 
+int
+eel_preferences_get_enum (const char *name)
+{
+	int               ret_val;
+	char             *str_value;
+	GConfValue       *value;
+	EelEnumeration   *enumeration;
+	PreferencesEntry *entry;
+
+	g_return_val_if_fail (name != NULL, 0);
+	g_return_val_if_fail (preferences_is_initialized (), 0);
+
+	entry = preferences_global_table_lookup_or_insert (name);
+	g_return_val_if_fail (entry != NULL, 0);
+
+	enumeration = eel_enumeration_lookup (entry->enumeration_id);
+
+	if (!enumeration) {
+		g_warning ("No enum entry for '%s' (%s)",
+			   name, entry->enumeration_id);
+		return 0;
+	}
+
+	value = preferences_get_value (name);
+	if (value->type == GCONF_VALUE_INT) { /* compatibility path */
+		ret_val = preferences_gconf_value_get_int (value);
+		eel_gconf_value_free (value);
+		return ret_val;
+	}
+
+	str_value = preferences_gconf_value_get_string (value);
+	eel_gconf_value_free (value);
+
+	if (str_value == NULL) {
+		g_warning ("No key for '%s' at %s", str_value, name);
+		return 0;
+	}
+
+	ret_val = eel_enumeration_get_sub_value (enumeration, str_value);
+	
+	g_free (str_value);
+
+	return ret_val;
+}
+
+void
+eel_preferences_set_enum (const char *name,
+			  int         int_value)
+{
+	const char       *str_value;
+	EelEnumeration   *enumeration;
+	PreferencesEntry *entry;
+
+	g_return_if_fail (name != NULL);
+	g_return_if_fail (preferences_is_initialized ());
+
+	entry = preferences_global_table_lookup_or_insert (name);
+	g_return_if_fail (entry != NULL);
+
+	enumeration = eel_enumeration_lookup (entry->enumeration_id);
+
+	if (!enumeration) {
+		g_warning ("No enum entry for '%s' (%s)",
+			   name, entry->enumeration_id);
+		return;
+	}
+
+	str_value = eel_enumeration_get_sub_name (enumeration, int_value);
+
+	if (str_value == NULL) {
+		g_warning ("No enum match for '%d'", int_value);
+		return;
+	}
+
+	eel_preferences_set (name, str_value);
+}
+
 void
 eel_preferences_set (const char *name,
 		     const char *string_value)
@@ -546,11 +622,18 @@ preferences_entry_update_auto_storage (PreferencesEntry *entry)
 
 	switch (entry->type) {
 	case PREFERENCE_STRING:
-		new_string_value = eel_preferences_get (entry->name);
-		g_list_foreach (entry->auto_storage_list,
-				update_auto_string,
-				new_string_value);
-		g_free (new_string_value);
+		if (entry->enumeration_id != NULL) {
+			new_int_value = eel_preferences_get_enum (entry->name);
+			g_list_foreach (entry->auto_storage_list,
+					update_auto_integer_or_boolean,
+					GINT_TO_POINTER (new_int_value));
+		} else {
+			new_string_value = eel_preferences_get (entry->name);
+			g_list_foreach (entry->auto_storage_list,
+					update_auto_string,
+					new_string_value);
+			g_free (new_string_value);
+		}
 		break;
 	case PREFERENCE_STRING_LIST:
 		new_string_list_value = eel_preferences_get_string_list (entry->name);
@@ -1023,6 +1106,28 @@ eel_preferences_add_auto_integer (const char *name,
 	update_auto_integer_or_boolean (storage, GINT_TO_POINTER (value));
 }
 
+
+void
+eel_preferences_add_auto_enum (const char *name,
+			       int *storage)
+{
+	PreferencesEntry *entry;
+	int value;
+
+	g_return_if_fail (name != NULL);
+	g_return_if_fail (storage != NULL);
+	g_return_if_fail (preferences_is_initialized ());
+
+	entry = preferences_global_table_lookup_or_insert (name);
+	g_assert (entry != NULL);
+	g_assert (entry->enumeration_id != NULL);
+
+	preferences_entry_add_auto_storage (entry, storage, PREFERENCE_STRING);
+
+	value = eel_preferences_get_enum (entry->name);
+	update_auto_integer_or_boolean (storage, GINT_TO_POINTER (value));
+}
+
 void
 eel_preferences_add_auto_boolean (const char *name,
 				  gboolean *storage)
@@ -1246,7 +1351,7 @@ eel_preferences_get_enumeration_id (const char *name)
 
 	entry = preferences_global_table_lookup_or_insert (name);
 
-	return entry->enumeration_id ? g_strdup (entry->enumeration_id) : NULL;
+	return g_strdup (entry->enumeration_id);
 }
 
 static void
@@ -1340,7 +1445,7 @@ eel_preferences_set_emergency_fallback_string_list (const char    *name,
 	gconf_value_set_list_type (gconf_value, GCONF_VALUE_STRING);
 	list = NULL;
 	eel_string_list_for_each (value, listify_strings_foreach, &list);
-	gconf_value_set_list_nocopy (gconf_value, list);
+	gconf_value_set_list_nocopy (gconf_value, g_slist_reverse (list));
 
 	preferences_set_emergency_fallback_stealing_value (name, gconf_value);
 }
