@@ -414,18 +414,24 @@ show_message_dialog (const char *message,
 		     const char *dialog_title,
 		     GtkMessageType type,
 		     GtkButtonsType buttons_type,
-		     const char *additional_button,
+		     const char *additional_button_label,
 		     int additional_button_response_id,
 		     GtkWindow *parent)
 {
 	GtkDialog *dialog;
+	GtkWidget *additional_button;
+	GtkWidget *box;
 
 	dialog = create_message_dialog (message, dialog_title, type, 
 					buttons_type, parent);
-	if (additional_button != NULL) {
-		gtk_dialog_add_button (dialog,
-				       additional_button,
-				       additional_button_response_id);
+	if (additional_button_label != NULL) {
+		additional_button = gtk_dialog_add_button (dialog,
+							   additional_button_label,
+							   additional_button_response_id);
+		box = gtk_widget_get_parent (additional_button);
+		if (GTK_IS_BOX (box)) {
+			gtk_box_reorder_child (GTK_BOX (box), additional_button, 0);
+		}
 	}
 	gtk_widget_show (GTK_WIDGET (dialog));
 
